@@ -1,9 +1,12 @@
 from typing import Optional
 
+from django import forms
 from django.contrib.auth import authenticate, get_user_model, login
 from django.http import HttpRequest
+from django.utils.translation import gettext_lazy as _
 
 from swap_user.helpers import (
+    check_user_was_banned,
     generate_otp,
     get_banned_for_invalid_login_cache_key,
     get_banned_for_otp_rate_limit_cache_key,
@@ -81,6 +84,13 @@ class GetOTPService:
             cache_key=rate_limit_cache_key, value=USER_IS_BANNED, expire=ban_timeout,
         )
 
+    def do_extra_logic(self, *, username: str, **kwargs):
+        """
+        Hook that can be used to provide some extra logic layer.
+        """
+
+        pass
+
     def _get_user(self, username: str) -> Optional[UserModel]:
         """
         Method, that handles user presence in our DB or not.
@@ -155,3 +165,88 @@ class CheckOTPService:
         set_key_to_cache(
             cache_key=banned_user_cache_key, value=USER_IS_BANNED, expire=ban_timeout,
         )
+
+    def do_extra_logic_on_valid(self, *, username: str, otp_password: str, **kwargs):
+        """
+        Hook that can be used to provide some extra logic layer on `form_valid`.
+        """
+
+        pass
+
+    def do_extra_logic_on_invalid(self, *, username: str, **kwargs):
+        """
+        Hook that can be used to provide some extra logic layer on `form_invalid`.
+        """
+
+        pass
+
+
+class ValidationService:
+    """
+    Service that holds different checks and validations.
+    """
+
+    def check_user_is_banned_for_otp_rate_limit(self, username: str):
+        """
+        We are banning user for too many sent OTP codes.
+        Here is a check.
+        """
+
+        cache_key = get_banned_for_otp_rate_limit_cache_key(username)
+        is_banned = check_user_was_banned(cache_key)
+
+        if is_banned:
+            message = _("You are banned - try again later.")
+            code = "banned"
+            raise forms.ValidationError(message, code)
+
+    def check_user_is_banned_for_invalid_login_attempts(self, username: str):
+        """
+        We are banning user for too many invalid login
+        attempts.
+        Here we are checking for this.
+        """
+
+        cache_key = get_banned_for_invalid_login_cache_key(username)
+        is_banned = check_user_was_banned(cache_key)
+
+        if is_banned:
+            message = _("You are banned - contact with admin.")
+            code = "banned"
+            raise forms.ValidationError(message, code)
+
+    def check_password(self, username: str, otp: str):
+        """
+        Check backend cached OTP with user provided OTP.
+        """
+
+        user = self._get_user(username)
+
+        if not user.check_password(otp):
+            message = _("Invalid credentials.")
+            code = "invalid_credentials"
+            raise forms.ValidationError(message, code)
+
+    def check_extra(self, *, username: str, otp: str = None, **kwargs):
+        """
+        Hook for providing extra checks.
+        """
+
+        pass
+
+    def _get_user(self, username: str) -> UserModel:
+        """
+        Check User presence in our DB or not.
+        """
+
+        username_field = UserModel.USERNAME_FIELD
+        query_data = {username_field: username}
+
+        try:
+            user = UserModel.objects.get(**query_data)
+        except UserModel.DoesNotExist:
+            message = _("Invalid credentials.")
+            code = "invalid_credentials"
+            raise forms.ValidationError(message, code)
+
+        return user
